==> ideaIC-173.3531.6-sources.jar: CallFrameView.kt <==
/*
 * Copyright 2000-2016 JetBrains s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.jetbrains.debugger.frame

import com.intellij.icons.AllIcons
import com.intellij.ui.ColoredTextContainer
import com.intellij.ui.SimpleTextAttributes
import com.intellij.xdebugger.evaluation.XDebuggerEvaluator
import com.intellij.xdebugger.frame.XCompositeNode
import com.intellij.xdebugger.frame.XStackFrame
import org.jetbrains.concurrency.Promise
import org.jetbrains.debugger.*

// isInLibraryContent call could be costly, so we compute it only once (our customizePresentation called on each repaint)
class CallFrameView @JvmOverloads constructor(val callFrame: CallFrame,
                                              override val viewSupport: DebuggerViewSupport,
                                              val script: Script? = null,
                                              sourceInfo: SourceInfo? = null,
                                              isInLibraryContent: Boolean? = null,
                                              override val vm: Vm? = null) : XStackFrame(), VariableContext {
  private val sourceInfo = sourceInfo ?: viewSupport.getSourceInfo(script, callFrame)
  private val isInLibraryContent: Boolean = isInLibraryContent ?: (this.sourceInfo != null && viewSupport.isInLibraryContent(this.sourceInfo, script))

  private var evaluator: XDebuggerEvaluator? = null

  override fun getEqualityObject() = callFrame.equalityObject

  override fun computeChildren(node: XCompositeNode) {
    node.setAlreadySorted(true)
    createAndAddScopeList(node, callFrame.variableScopes, this, callFrame)
  }

  override val evaluateContext: EvaluateContext
    get() = callFrame.evaluateContext

  override fun watchableAsEvaluationExpression() = true

  override val memberFilter: Promise<MemberFilter>
    get() = viewSupport.getMemberFilter(this)

  fun getMemberFilter(scope: Scope) = createVariableContext(scope, this, callFrame).memberFilter

  override fun getEvaluator(): XDebuggerEvaluator? {
    if (evaluator == null) {
      evaluator = viewSupport.createFrameEvaluator(this)
    }
    return evaluator
  }

  override fun getSourcePosition() = sourceInfo

  override fun customizePresentation(component: ColoredTextContainer) {
    if (sourceInfo == null) {
      val scriptName = if (script == null) "unknown" else script.url.trimParameters().toDecodedForm()
      val line = callFrame.line
      component.append(if (line == -1) scriptName else "$scriptName:$line", SimpleTextAttributes.ERROR_ATTRIBUTES)
      return
    }

    val fileName = sourceInfo.file.name
    val line = sourceInfo.line + 1

    val textAttributes =
        if (isInLibraryContent || callFrame.isFromAsyncStack) SimpleTextAttributes.GRAYED_ATTRIBUTES
        else SimpleTextAttributes.REGULAR_ATTRIBUTES

    val functionName = sourceInfo.functionName
    if (functionName == null || (functionName.isEmpty() && callFrame.hasOnlyGlobalScope)) {
      if (fileName.startsWith("index.")) {
        sourceInfo.file.parent?.let {
          component.append("${it.name}/", textAttributes)
        }
      }
      component.append("$fileName:$line", textAttributes)
    }
    else {
      if (functionName.isEmpty()) {
        component.append("anonymous", if (isInLibraryContent) SimpleTextAttributes.GRAYED_ITALIC_ATTRIBUTES else SimpleTextAttributes.REGULAR_ITALIC_ATTRIBUTES)
      }
      else {
        component.append(functionName, textAttributes)
      }
      component.append("(), $fileName:$line", textAttributes)
    }
    component.setIcon(AllIcons.Debugger.StackFrame)
  }
}

==> ideaIC-182.3208.16-sources.jar: CallFrameView.kt <==
/*
 * Copyright 2000-2016 JetBrains s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.jetbrains.debugger.frame

import com.intellij.icons.AllIcons
import com.intellij.ui.ColoredTextContainer
import com.intellij.ui.SimpleTextAttributes
import com.intellij.xdebugger.evaluation.XDebuggerEvaluator
import com.intellij.xdebugger.frame.XCompositeNode
import com.intellij.xdebugger.frame.XStackFrame
import org.jetbrains.concurrency.Promise
import org.jetbrains.debugger.*

// isInLibraryContent call could be costly, so we compute it only once (our customizePresentation called on each repaint)
class CallFrameView @JvmOverloads constructor(val callFrame: CallFrame,
                                              override val viewSupport: DebuggerViewSupport,
                                              val script: Script? = null,
                                              sourceInfo: SourceInfo? = null,
                                              isInLibraryContent: Boolean? = null,
                                              override val vm: Vm? = null) : XStackFrame(), VariableContext {
  private val sourceInfo = sourceInfo ?: viewSupport.getSourceInfo(script, callFrame)
  private val isInLibraryContent: Boolean = isInLibraryContent ?: (this.sourceInfo != null && viewSupport.isInLibraryContent(this.sourceInfo, script))

  private var evaluator: XDebuggerEvaluator? = null

  override fun getEqualityObject(): Any = callFrame.equalityObject

  override fun computeChildren(node: XCompositeNode) {
    node.setAlreadySorted(true)
    createAndAddScopeList(node, callFrame.variableScopes, this, callFrame)
  }

  override val evaluateContext: EvaluateContext
    get() = callFrame.evaluateContext

  override fun watchableAsEvaluationExpression(): Boolean = true

  override val memberFilter: Promise<MemberFilter>
    get() = viewSupport.getMemberFilter(this)

  fun getMemberFilter(scope: Scope): Promise<MemberFilter> = createVariableContext(scope, this, callFrame).memberFilter

  override fun getEvaluator(): XDebuggerEvaluator? {
    if (evaluator == null) {
      evaluator = viewSupport.createFrameEvaluator(this)
    }
    return evaluator
  }

  override fun getSourcePosition(): SourceInfo? = sourceInfo

  override fun customizePresentation(component: ColoredTextContainer) {
    if (sourceInfo == null) {
      val scriptName = if (script == null) "unknown" else script.url.trimParameters().toDecodedForm()
      val line = callFrame.line
      component.append(if (line == -1) scriptName else "$scriptName:$line", SimpleTextAttributes.ERROR_ATTRIBUTES)
      return
    }

    val fileName = sourceInfo.file.name
    val line = sourceInfo.line + 1

    val textAttributes =
        if (isInLibraryContent || callFrame.isFromAsyncStack) SimpleTextAttributes.GRAYED_ATTRIBUTES
        else SimpleTextAttributes.REGULAR_ATTRIBUTES

    val functionName = sourceInfo.functionName
    if (functionName == null || (functionName.isEmpty() && callFrame.hasOnlyGlobalScope)) {
      if (fileName.startsWith("index.")) {
        sourceInfo.file.parent?.let {
          component.append("${it.name}/", textAttributes)
        }
      }
      component.append("$fileName:$line", textAttributes)
    }
    else {
      if (functionName.isEmpty()) {
        component.append("anonymous", if (isInLibraryContent) SimpleTextAttributes.GRAYED_ITALIC_ATTRIBUTES else SimpleTextAttributes.REGULAR_ITALIC_ATTRIBUTES)
      }
      else {
        component.append(functionName, textAttributes)
      }
      component.append("(), $fileName:$line", textAttributes)
    }
    component.setIcon(AllIcons.Debugger.StackFrame)
  }
}
